﻿using SfSchedule_NavigationButton.Models;
using Syncfusion.UI.Xaml.Schedule;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Threading.Tasks;
using System.Windows.Input;

namespace SfSchedule_NavigationButton
{
    public class AppointmentCalendarViewModel : BaseViewModel
    {
        private readonly List<DateTime> _loadedAppointmentDates;
        private readonly DragSnapInConfiguration _dragSnapInConfiguration;

        public bool IsLoadingAppointments { get; set; }

        public ObservableCollection<Appointment> Appointments { get; set; }

        public ObservableCollection<DateTime> ScheduleSelectedDates { get; set; }

        public ScheduleType CurrentScheduleType { get; set; }

        public bool IsDayMode => CurrentScheduleType == ScheduleType.Day;

        public bool IsWorkWeekMode => CurrentScheduleType == ScheduleType.WorkWeek;

        public bool IsMonthMode => CurrentScheduleType == ScheduleType.Month;

        public bool IsTimeLineMode => CurrentScheduleType == ScheduleType.TimeLine;

        public ICommand SetPickupScheduleTypeCommand { get; set; }

        public ICommand ScheduleVisibleDatesChangingCommand { get; set; }

        public ICommand ScheduleAppointmentEditorOpeningCommand { get; set; }

        public ICommand ScheduleAppointmentStartResizingCommand { get; set; }

        public ICommand ScheduleAppointmentEndDraggingCommand { get; set; }

        public ICommand ScheduleContextMenuOpeningCommand { get; set; }

        public ICommand NavigateTodayCommand { get; set; }

        public ICommand OpenEditAppointmentWindowCommand { get; set; }

        public ICommand RemoveAppointmentCommand { get; set; }

        public ICommand ReloadAppointmentsCommand { get; set; }

        public AppointmentCalendarViewModel()
        {
            InitializeCommands();

            _loadedAppointmentDates = new List<DateTime>();
            Appointments = new ObservableCollection<Appointment>();

            CurrentScheduleType = ScheduleType.WorkWeek;

            _dragSnapInConfiguration = DragSnapInConfiguration.SnapToPreviousIntervall;

            Appointments.Add(new Appointment
            {
                Start = DateTime.Today.Add(new TimeSpan(10, 30, 0)),
                End = DateTime.Today.Add(new TimeSpan(11, 30, 0)),
            });

            Appointments.Add(new Appointment
            {
                Start = DateTime.Today.Add(new TimeSpan(1, 10, 30, 0)),
                End = DateTime.Today.Add(new TimeSpan(1, 11, 30, 0)),
            });

            Appointments.Add(new Appointment
            {
                Start = DateTime.Today.Add(new TimeSpan(2, 10, 30, 0)),
                End = DateTime.Today.Add(new TimeSpan(2, 11, 30, 0)),
            });

            Appointments.Add(new Appointment
            {
                Start = DateTime.Today.Add(new TimeSpan(3, 10, 30, 0)),
                End = DateTime.Today.Add(new TimeSpan(3, 11, 30, 0)),
            });
        }

        private void InitializeCommands()
        {
            SetPickupScheduleTypeCommand = new ParameterizedRelayCommand<ScheduleType>(SetPickupScheduleType);

            ScheduleVisibleDatesChangingCommand = new ParameterizedRelayCommand<VisibleDatesChangingEventArgs>(VisibleDatesChanging);
            ScheduleAppointmentEditorOpeningCommand = new ParameterizedRelayCommand<AppointmentEditorOpeningEventArgs>(ScheduleAppointmentEditorOpening);
            ScheduleAppointmentEndDraggingCommand = new ParameterizedRelayCommand<AppointmentEndDraggingEventArgs>(ScheduleAppointmentEndDragging);
            ScheduleAppointmentStartResizingCommand = new ParameterizedRelayCommand<AppointmentStartResizingEventArgs>(ScheduleAppointmentStartResizing);
            ScheduleContextMenuOpeningCommand = new ParameterizedRelayCommand<ContextMenuOpeningEventArgs>(ScheduleContextMenuOpening);

            NavigateTodayCommand = new RelayCommand(NavigateToday);

            OpenEditAppointmentWindowCommand = new ParameterizedRelayCommand<Appointment>(OpenEditAppointmentWindow);
            RemoveAppointmentCommand = new ParameterizedRelayCommand<Appointment>(RemoveAppointment);
            ReloadAppointmentsCommand = new RelayCommand(ReloadAppointments);
        }

        private void OpenEditAppointmentWindow(Appointment appointment)
        {
            
        }

        private void ReloadAppointments()
        {
            ClearAppointmentCache();

            var schedule = ((AppointmentCalendar)WindowOrUserControl).AppointmentSchedule;

            schedule.MoveToDate(schedule.VisibleDates[0]);
        }

        private void ClearAppointmentCache()
        {
            _loadedAppointmentDates.Clear();
            Appointments.Clear();
        }

        private async void RemoveAppointment(Appointment appointment)
        {
           
        }

        private void NavigateToday()
        {
            var schedule = ((AppointmentCalendar)WindowOrUserControl).AppointmentSchedule;

            schedule.MoveToDate(DateTime.Today);
        }

        private void SetPickupScheduleType(ScheduleType scheduleType)
        {
            CurrentScheduleType = scheduleType;
        }

        private void ScheduleAppointmentEndDragging(AppointmentEndDraggingEventArgs e)
        {
           
            e.Cancel = true;
        }

        private async void VisibleDatesChanging(VisibleDatesChangingEventArgs e)
        {
            var dates = e.NewValue as ObservableCollection<DateTime>;

            await LoadVisibleDatesAsync(dates);
        }

        private void ScheduleAppointmentStartResizing(AppointmentStartResizingEventArgs e)
        {
            e.Cancel = true;
        }

        private void ScheduleContextMenuOpening(ContextMenuOpeningEventArgs e)
        {
            e.Cancel = true;
        }

        private void ScheduleAppointmentEditorOpening(AppointmentEditorOpeningEventArgs e)
        {
            e.Cancel = true;
        }

     

        

        private async Task LoadVisibleDatesAsync(ObservableCollection<DateTime> visibleDates)
        {
            //ObservableCollection<DateTime> datesToLoad = new ObservableCollection<DateTime>();

            //foreach (var visibleDate in visibleDates)
            //{
            //    if (_loadedAppointmentDates.Any(d => d.Date == visibleDate.Date))
            //    {
            //        continue;
            //    }

            //    datesToLoad.Add(visibleDate.Date);
            //    _loadedAppointmentDates.Add(visibleDate.Date);
            //}

            //if (datesToLoad.Count > 0)
            //{
            //    IsLoadingAppointments = true;

            //    var loadedAppointments = await AppointmentManager.GetAppointmentsForDatesAsync(datesToLoad);

            //    loadedAppointments.ForEach(loadedAppointment => Appointments.Add(loadedAppointment));
            //    IsLoadingAppointments = false;
            //}
        }
    }
}