﻿using SfSchedule_NavigationButton.Models.Enums;
using System;
using System.Collections.ObjectModel;
using System.Linq;

namespace SfSchedule_NavigationButton.Models
{
    public class Order : ObservableObject
    {        
        public int Id { get; set; }        

        public DateTime StartDate { get; set; }

        public DateTime EndDate { get; set; }

        public int Duration => (EndDate.Date - StartDate.Date).Days + 1;

        public Customer Customer { get; set; }

        public bool Paid { get; set; }

        public ObservableCollection<OrderItem> OrderItems { get; set; }

        public ObservableCollection<Appointment> Appointments { get; set; }

        public Appointment PickupAppointment => GetPickupAppointment();

        public Appointment ReturnAppointment => GetReturnAppointment();

        public InventarStatus MostImportantInventarStatus { get; set; }

        public bool HasMostImportantInventarStatus => MostImportantInventarStatus != null;

        public double CustomPrice { get; set; }

        public double MontagePrice { get; set; }

        public string Note { get; set; }

        public bool HasNote => !String.IsNullOrWhiteSpace(Note);

        public DateTime CreateTime { get; set; }

        public Order()
        {
            Note = "";
            Appointments = new ObservableCollection<Appointment>();
            OrderItems = new ObservableCollection<OrderItem>();
        }

        private Appointment GetPickupAppointment()
        {
            return Appointments.FirstOrDefault(appointment => appointment.Type == AppointmentType.Pickup);
        }

        private Appointment GetReturnAppointment()
        {
            return Appointments.FirstOrDefault(appointment => appointment.Type == AppointmentType.Return);
        }
    }
}
